@@ -5,7 +5,7 @@
  * Copyright (c) 2014-2021 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2015-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2018-2022 Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2018-2025 Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * Copyright (c) 2020-2023 Triad National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
@@ -823,20 +823,17 @@ select_prov:
         }
     } else {
         *accelerator_support = true;
-        ompi_mtl_ofi.hmem_needs_reg = true;
-        /*
-         * Workaround for the fact that the CXI provider actually doesn't need for accelerator memory to be registered
-         * for local buffers, but if one does do so using fi_mr_regattr, one actually needs to manage the
-         * requested_key field in the fi_mr_attr attr argument, and the OFI MTL doesn't track which requested_keys
-         * have already been registered. So just set a flag to disable local registration.  Note the OFI BTL doesn't
-         * have a problem here since it uses fi_mr_regattr only within the context of an rcache, and manages the
-         * requested_key field in this way.
-         */
-         if ((NULL != strstr(prov->fabric_attr->prov_name, "cxi")) ||
-             (NULL != strstr(prov->fabric_attr->prov_name, "CXI")) ) {
-             ompi_mtl_ofi.hmem_needs_reg = false;
-         }
 
+        /* Only explicitly register domain buffers if the provider requires it.
+           For example, CXI does not require it but EFA does require it. */
+        if ((prov->domain_attr->mr_mode & FI_MR_HMEM) != 0) {
+            ompi_mtl_ofi.hmem_needs_reg = true;
+            opal_output_verbose(50, opal_common_ofi.output,
+                                "Support for device buffers enabled with explicit registration");
+        } else {
+            opal_output_verbose(50, opal_common_ofi.output,
+                                "Support for device buffers enabled with implicit registration");
+        }
     }
 #else
     opal_output_verbose(50, opal_common_ofi.output,
@@ -844,6 +841,10 @@ select_prov:
                         __FILE__, __LINE__);
 #endif
 
+    if (ompi_mtl_ofi.hmem_needs_reg) {
+        ompi_mtl_ofi_rcache_init();
+    }
+
     /**
      * Select the format of the OFI tag
      */
@@ -982,17 +983,6 @@ select_prov:
         goto error;
     }
 
-    /**
-     * Unfortunately the attempt to implement FI_MR_SCALABLE in the GNI provider
-     * doesn't work, at least not well.  Since we're asking for the 1.5 libfabric
-     * API now, we have to tell GNI we want to use Mr. Basic.  Using FI_MR_BASIC
-     * rather than FI_MR_VIRT_ADDR | FI_MR_ALLOCATED | FI_MR_PROV_KEY to stay
-     * compatible with older libfabrics.
-     */
-    if (!strncmp(prov->fabric_attr->prov_name,"gni",3)) {
-         prov->domain_attr->mr_mode = FI_MR_BASIC;
-    }
-
     /**
      * Create the access domain, which is the physical or virtual network or
      * hardware port/collection of ports.  Returns a domain object that can be
@@ -1191,6 +1181,11 @@ ompi_mtl_ofi_finalize(struct mca_mtl_base_module_t *mtl)
 {
     ssize_t ret;
 
+    if (NULL != ompi_mtl_ofi.rcache) {
+        mca_rcache_base_module_destroy(ompi_mtl_ofi.rcache);
+        ompi_mtl_ofi.rcache = NULL;
+    }
+
     opal_progress_unregister(ompi_mtl_ofi_progress_no_inline);
 
     /* Close all the OFI objects */
